@@ -45,6 +45,14 @@ std::string OptionalsToString(const Connection *conn, Optionals<T> &opts) {
   return str;
 }
 
+std::string SizeToString(const std::vector<std::size_t> &elems) {
+  std::string result = MultiLen(elems.size());
+  for (const auto &elem : elems) {
+    result += redis::Integer(elem);
+  }
+  return result;
+}
+
 class CommandJsonSet : public Commander {
  public:
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
@@ -383,8 +391,12 @@ class CommandJsonObjLen : public Commander {
     Optionals<uint64_t> results;
     auto s = json.ObjLen(args_[1], path, &results);
     if (s.IsNotFound()) {
-      *output = conn->NilString();
-      return Status::OK();
+      if (args_.size() == 2) {
+        *output = conn->NilString();
+        return Status::OK();
+      } else {
+        return {Status::RedisExecErr, "Path '" + args_[2] + "' does not exist or not an object"};
+      }
     }
     if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
 
@@ -411,8 +423,7 @@ class CommandJsonArrTrim : public Commander {
     auto s = json.ArrTrim(args_[1], path_, start_, stop_, &results);
 
     if (s.IsNotFound()) {
-      *output = conn->NilString();
-      return Status::OK();
+      return {Status::RedisExecErr, "could not perform this operation on a key that doesn't exist"};
     }
     if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
 
@@ -559,6 +570,15 @@ class CommandJsonStrLen : public Commander {
 
     Optionals<uint64_t> results;
     auto s = json.StrLen(args_[1], path, &results);
+    if (s.IsNotFound()) {
+      if (args_.size() == 2) {
+        *output = conn->NilString();
+        return Status::OK();
+      } else {
+        return {Status::RedisExecErr, "could not perform this operation on a key that doesn't exist"};
+      }
+    }
+
     if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
 
     *output = OptionalsToString(conn, results);
@@ -594,6 +614,65 @@ class CommandJsonMGet : public Commander {
   }
 };
 
+class CommandJsonMSet : public Commander {
+ public:
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    if ((args_.size() - 1) % 3 != 0) {
+      return {Status::RedisExecErr, errWrongNumOfArguments};
+    }
+
+    redis::Json json(svr->storage, conn->GetNamespace());
+
+    std::vector<std::string> user_keys;
+    std::vector<std::string> paths;
+    std::vector<std::string> values;
+    for (size_t i = 0; i < (args_.size() - 1) / 3; i++) {
+      user_keys.emplace_back(args_[i * 3 + 1]);
+      paths.emplace_back(args_[i * 3 + 2]);
+      values.emplace_back(args_[i * 3 + 3]);
+    }
+
+    if (auto s = json.MSet(user_keys, paths, values); !s.ok()) return {Status::RedisExecErr, s.ToString()};
+
+    *output = redis::SimpleString("OK");
+    return Status::OK();
+  }
+};
+
+class CommandJsonDebug : public Commander {
+ public:
+  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+    redis::Json json(svr->storage, conn->GetNamespace());
+
+    std::string path = "$";
+
+    if (!util::EqualICase(args_[1], "memory")) {
+      return {Status::RedisExecErr, "ERR wrong number of arguments for 'json.debug' command"};
+    }
+
+    if (args_.size() == 4) {
+      path = args_[3];
+    } else if (args_.size() > 4) {
+      return {Status::RedisExecErr, "The number of arguments is more than expected"};
+    }
+
+    std::vector<std::size_t> results;
+    auto s = json.DebugMemory(args_[2], path, &results);
+
+    if (s.IsNotFound()) {
+      if (args_.size() == 3) {
+        *output = redis::Integer(0);
+      } else {
+        *output = SizeToString(results);
+      }
+      return Status::OK();
+    }
+    if (!s.ok()) return {Status::RedisExecErr, s.ToString()};
+
+    *output = SizeToString(results);
+    return Status::OK();
+  }
+};
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandJsonSet>("json.set", 4, "write", 1, 1, 1),
                         MakeCmdAttr<CommandJsonGet>("json.get", -2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandJsonInfo>("json.info", 2, "read-only", 1, 1, 1),
@@ -616,6 +695,8 @@ REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandJsonSet>("json.set", 4, "write", 1, 1
                         MakeCmdAttr<CommandJsonObjLen>("json.objlen", -2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandJsonStrAppend>("json.strappend", -3, "write", 1, 1, 1),
                         MakeCmdAttr<CommandJsonStrLen>("json.strlen", -2, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandJsonMGet>("json.mget", -3, "read-only", 1, 1, 1), );
+                        MakeCmdAttr<CommandJsonMGet>("json.mget", -3, "read-only", 1, -2, 1),
+                        MakeCmdAttr<CommandJsonMSet>("json.mset", -4, "write", 1, -3, 3),
+                        MakeCmdAttr<CommandJsonDebug>("json.debug", -3, "read-only", 2, 2, 1));
 
 }  // namespace redis
