@@ -31,8 +31,7 @@ class CommandEvalImpl : public Commander {
  public:
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     if (evalsha && args_[1].size() != 40) {
-      *output = redis::Error(errNoMatchingScript);
-      return Status::OK();
+      return {Status::RedisNoScript, errNoMatchingScript};
     }
 
     int64_t numkeys = GET_OR_RET(ParseInt<int64_t>(args_[2], 10));
@@ -116,6 +115,14 @@ CommandKeyRange GetScriptEvalKeyRange(const std::vector<std::string> &args) {
   return {3, 2 + numkeys, 1};
 }
 
+uint64_t GenerateScriptFlags(uint64_t flags, const std::vector<std::string> &args) {
+  if (util::EqualICase(args[1], "load") || util::EqualICase(args[1], "flush")) {
+    return flags | kCmdWrite;
+  }
+
+  return flags;
+}
+
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandEval>("eval", -3, "exclusive write no-script", GetScriptEvalKeyRange),
                         MakeCmdAttr<CommandEvalSHA>("evalsha", -3, "exclusive write no-script", GetScriptEvalKeyRange),
                         MakeCmdAttr<CommandEvalRO>("eval_ro", -3, "read-only no-script ro-script",
