@@ -18,6 +18,8 @@
  *
  */
 
+#include <algorithm>
+#include <limits>
 #include <memory>
 #include <stdexcept>
 
@@ -31,6 +33,39 @@
 
 namespace redis {
 
+class CommandXAck : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    stream_name_ = args[1];
+    group_name_ = args[2];
+    StreamEntryID tmp_id;
+    for (size_t i = 3; i < args.size(); ++i) {
+      auto s = ParseStreamEntryID(args[i], &tmp_id);
+      if (!s.IsOK()) return s;
+      entry_ids_.emplace_back(tmp_id);
+    }
+
+    return Status::OK();
+  }
+
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+    uint64_t acknowledged = 0;
+    auto s = stream_db.DeletePelEntries(stream_name_, group_name_, entry_ids_, &acknowledged);
+    if (!s.ok()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+    *output = redis::Integer(acknowledged);
+
+    return Status::OK();
+  }
+
+ private:
+  std::string stream_name_;
+  std::string group_name_;
+  std::vector<StreamEntryID> entry_ids_;
+};
+
 class CommandXAdd : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
@@ -95,9 +130,7 @@ class CommandXAdd : public Commander {
         }
 
         auto s = ParseStreamEntryID(args[min_id_idx], &min_id_);
-        if (!s.IsOK()) {
-          return {Status::RedisParseErr, s.Msg()};
-        }
+        if (!s.IsOK()) return s;
 
         with_min_id_ = true;
         i += eq_sign_found ? 3 : 2;
@@ -110,9 +143,7 @@ class CommandXAdd : public Commander {
 
       if (!entry_id_found) {
         auto result = ParseNextStreamEntryIDStrategy(val);
-        if (!result.IsOK()) {
-          return {Status::RedisParseErr, result.Msg()};
-        }
+        if (!result.IsOK()) return result;
 
         next_id_strategy_ = std::move(*result);
 
@@ -207,6 +238,219 @@ class CommandXDel : public Commander {
   std::vector<redis::StreamEntryID> ids_;
 };
 
+class CommandXClaim : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    if (args.size() < 6) {
+      return {Status::RedisParseErr, errWrongNumOfArguments};
+    }
+
+    CommandParser parser(args, 1);
+    stream_name_ = GET_OR_RET(parser.TakeStr());
+    group_name_ = GET_OR_RET(parser.TakeStr());
+    consumer_name_ = GET_OR_RET(parser.TakeStr());
+    auto parse_result = parser.TakeInt<int64_t>();
+    if (!parse_result.IsOK()) {
+      return {Status::RedisParseErr, errValueNotInteger};
+    }
+    min_idle_time_ms_ = parse_result.GetValue();
+    if (min_idle_time_ms_ < 0) {
+      min_idle_time_ms_ = 0;
+    }
+
+    while (parser.Good() && !isOption(parser.RawPeek())) {
+      auto raw_id = GET_OR_RET(parser.TakeStr());
+      redis::StreamEntryID id;
+      auto s = ParseStreamEntryID(raw_id, &id);
+      if (!s.IsOK()) {
+        return s;
+      }
+      entry_ids_.emplace_back(id);
+    }
+
+    while (parser.Good()) {
+      if (parser.EatEqICase("idle")) {
+        auto parse_result = parser.TakeInt<int64_t>();
+        if (!parse_result.IsOK()) {
+          return {Status::RedisParseErr, errValueNotInteger};
+        }
+        if (parse_result.GetValue() < 0) {
+          return {Status::RedisParseErr, "IDLE for XCLAIM must be non-negative"};
+        }
+        stream_claim_options_.idle_time_ms = parse_result.GetValue();
+      } else if (parser.EatEqICase("time")) {
+        auto parse_result = parser.TakeInt<int64_t>();
+        if (!parse_result.IsOK()) {
+          return {Status::RedisParseErr, errValueNotInteger};
+        }
+        if (parse_result.GetValue() < 0) {
+          return {Status::RedisParseErr, "TIME for XCLAIM must be non-negative"};
+        }
+        stream_claim_options_.with_time = true;
+        stream_claim_options_.last_delivery_time_ms = parse_result.GetValue();
+      } else if (parser.EatEqICase("retrycount")) {
+        auto parse_result = parser.TakeInt<int64_t>();
+        if (!parse_result.IsOK()) {
+          return {Status::RedisParseErr, errValueNotInteger};
+        }
+        if (parse_result.GetValue() < 0) {
+          return {Status::RedisParseErr, "RETRYCOUNT for XCLAIM must be non-negative"};
+        }
+        stream_claim_options_.with_retry_count = true;
+        stream_claim_options_.last_delivery_count = parse_result.GetValue();
+      } else if (parser.EatEqICase("force")) {
+        stream_claim_options_.force = true;
+      } else if (parser.EatEqICase("justid")) {
+        stream_claim_options_.just_id = true;
+      } else if (parser.EatEqICase("lastid")) {
+        auto last_id = GET_OR_RET(parser.TakeStr());
+        auto s = ParseStreamEntryID(last_id, &stream_claim_options_.last_delivered_id);
+        if (!s.IsOK()) {
+          return s;
+        }
+      } else {
+        return parser.InvalidSyntax();
+      }
+    }
+    return Status::OK();
+  }
+
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+    StreamClaimResult result;
+    auto s = stream_db.ClaimPelEntries(stream_name_, group_name_, consumer_name_, min_idle_time_ms_, entry_ids_,
+                                       stream_claim_options_, &result);
+    if (!s.ok()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+
+    if (s.IsNotFound()) {
+      return {Status::RedisExecErr, errNoSuchKey};
+    }
+
+    if (!stream_claim_options_.just_id) {
+      output->append(redis::MultiLen(result.entries.size()));
+
+      for (const auto &e : result.entries) {
+        output->append(redis::MultiLen(2));
+        output->append(redis::BulkString(e.key));
+        output->append(conn->MultiBulkString(e.values));
+      }
+    } else {
+      output->append(redis::MultiLen(result.ids.size()));
+      for (const auto &id : result.ids) {
+        output->append(redis::BulkString(id));
+      }
+    }
+
+    return Status::OK();
+  }
+
+ private:
+  std::string stream_name_;
+  std::string group_name_;
+  std::string consumer_name_;
+  uint64_t min_idle_time_ms_;
+  std::vector<StreamEntryID> entry_ids_;
+  StreamClaimOptions stream_claim_options_;
+
+  bool static isOption(const std::string &arg) {
+    static const std::unordered_set<std::string> options = {"idle", "time", "retrycount", "force", "justid", "lastid"};
+    return options.find(util::ToLower(arg)) != options.end();
+  }
+};
+
+class CommandAutoClaim : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    CommandParser parser(args, 1);
+    key_name_ = GET_OR_RET(parser.TakeStr());
+    group_name_ = GET_OR_RET(parser.TakeStr());
+    consumer_name_ = GET_OR_RET(parser.TakeStr());
+    if (auto parse_status = parser.TakeInt<uint64_t>(); !parse_status.IsOK()) {
+      return {Status::RedisParseErr, "Invalid min-idle-time argument for XAUTOCLAIM"};
+    } else {
+      options_.min_idle_time_ms = parse_status.GetValue();
+    }
+
+    auto start_str = GET_OR_RET(parser.TakeStr());
+    if (!start_str.empty() && start_str.front() == '(') {
+      options_.exclude_start = true;
+      start_str = start_str.substr(1);
+    }
+    if (!options_.exclude_start && start_str == "-") {
+      options_.start_id = StreamEntryID::Minimum();
+    } else {
+      auto parse_status = ParseRangeStart(start_str, &options_.start_id);
+      if (!parse_status.IsOK()) {
+        return parse_status;
+      }
+    }
+
+    if (parser.EatEqICase("count")) {
+      uint64_t count = GET_OR_RET(parser.TakeInt<uint64_t>());
+      constexpr uint64_t min_count = 1;
+      uint64_t max_count = std::numeric_limits<int64_t>::max() /
+                           (std::max(static_cast<uint64_t>(sizeof(StreamEntryID)), options_.attempts_factors));
+      if (count < min_count || count > max_count) {
+        return {Status::RedisParseErr, "COUNT must be > 0"};
+      }
+      options_.count = count;
+    }
+
+    if (parser.Good() && parser.EatEqICase("justid")) {
+      options_.just_id = true;
+    }
+
+    return Status::OK();
+  }
+
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+    StreamAutoClaimResult result;
+    auto s = stream_db.AutoClaim(key_name_, group_name_, consumer_name_, options_, &result);
+    if (!s.ok()) {
+      if (s.IsNotFound()) {
+        return {Status::RedisExecErr,
+                "NOGROUP No such key '" + key_name_ + "' or consumer group '" + group_name_ + "'"};
+      }
+      return {Status::RedisExecErr, s.ToString()};
+    }
+    return sendResults(conn, result, output);
+  }
+
+ private:
+  Status sendResults(Connection *conn, const StreamAutoClaimResult &result, std::string *output) const {
+    output->append(redis::MultiLen(3));
+    output->append(redis::BulkString(result.next_claim_id));
+    output->append(redis::MultiLen(result.entries.size()));
+    for (const auto &item : result.entries) {
+      if (options_.just_id) {
+        output->append(redis::BulkString(item.key));
+      } else {
+        output->append(redis::MultiLen(2));
+        output->append(redis::BulkString(item.key));
+        output->append(redis::MultiLen(item.values.size()));
+        for (const auto &value_item : item.values) {
+          output->append(redis::BulkString(value_item));
+        }
+      }
+    }
+
+    output->append(redis::MultiLen(result.deleted_ids.size()));
+    for (const auto &item : result.deleted_ids) {
+      output->append(redis::BulkString(item));
+    }
+
+    return Status::OK();
+  }
+
+  std::string key_name_;
+  std::string group_name_;
+  std::string consumer_name_;
+  StreamAutoClaimOptions options_;
+};
+
 class CommandXGroup : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
@@ -250,7 +494,7 @@ class CommandXGroup : public Commander {
       return Status::OK();
     }
 
-    if (subcommand_ == "createconsumer") {
+    if (subcommand_ == "createconsumer" || subcommand_ == "delconsumer") {
       if (args.size() != 5) {
         return {Status::RedisParseErr, errWrongNumOfArguments};
       }
@@ -322,6 +566,16 @@ class CommandXGroup : public Commander {
       *output = redis::Integer(created_number);
     }
 
+    if (subcommand_ == "delconsumer") {
+      uint64_t deleted_pel = 0;
+      auto s = stream_db.DestroyConsumer(stream_name_, group_name_, consumer_name_, deleted_pel);
+      if (!s.ok()) {
+        return {Status::RedisExecErr, s.ToString()};
+      }
+
+      *output = redis::Integer(deleted_pel);
+    }
+
     if (subcommand_ == "setid") {
       auto s = stream_db.GroupSetId(stream_name_, group_name_, xgroup_create_options_);
       if (!s.ok()) {
@@ -401,10 +655,10 @@ class CommandXInfo : public Commander {
 
         count_ = *parse_result;
       }
-    } else if (val == "groups" && args.size() == 3) {
-      subcommand_ = "groups";
-    } else if (val == "consumers" && args.size() == 4) {
-      subcommand_ = "consumers";
+      // } else if (val == "groups" && args.size() == 3) {
+      //   subcommand_ = "groups";
+      // } else if (val == "consumers" && args.size() == 4) {
+      //   subcommand_ = "consumers";
     } else {
       return {Status::RedisParseErr, errUnknownSubcommandOrWrongArguments};
     }
@@ -543,7 +797,7 @@ class CommandXInfo : public Commander {
     }
 
     output->append(redis::MultiLen(result_vector.size()));
-    auto now = util::GetTimeStampMS();
+    auto now_ms = util::GetTimeStampMS();
     for (auto const &it : result_vector) {
       output->append(conn->HeaderOfMap(4));
       output->append(redis::BulkString("name"));
@@ -551,9 +805,9 @@ class CommandXInfo : public Commander {
       output->append(redis::BulkString("pending"));
       output->append(redis::Integer(it.second.pending_number));
       output->append(redis::BulkString("idle"));
-      output->append(redis::Integer(now - it.second.last_idle));
+      output->append(redis::Integer(now_ms - it.second.last_attempted_interaction_ms));
       output->append(redis::BulkString("inactive"));
-      output->append(redis::Integer(now - it.second.last_active));
+      output->append(redis::Integer(now_ms - it.second.last_successful_interaction_ms));
     }
 
     return Status::OK();
@@ -963,7 +1217,7 @@ class CommandXRead : public Commander,
       std::vector<StreamEntry> result;
       auto s = stream_db.Range(streams_[i], options, &result);
       if (!s.ok() && !s.IsNotFound()) {
-        conn_->Reply(redis::Error("ERR " + s.ToString()));
+        conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
         return;
       }
 
@@ -1036,6 +1290,303 @@ class CommandXRead : public Commander,
   void unblockAll() { srv_->UnblockOnStreams(streams_, conn_); }
 };
 
+class CommandXReadGroup : public Commander,
+                          private EvbufCallbackBase<CommandXReadGroup, false>,
+                          private EventCallbackBase<CommandXReadGroup> {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    size_t streams_word_idx = 0;
+    if (util::ToLower(args[1]) != "group") {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+    group_name_ = args[2];
+    consumer_name_ = args[3];
+
+    for (size_t i = 4; i < args.size();) {
+      auto arg = util::ToLower(args[i]);
+
+      if (arg == "streams") {
+        streams_word_idx = i;
+        break;
+      }
+
+      if (arg == "count") {
+        if (i + 1 >= args.size()) {
+          return {Status::RedisParseErr, errInvalidSyntax};
+        }
+
+        with_count_ = true;
+
+        auto parse_result = ParseInt<uint64_t>(args[i + 1], 10);
+        if (!parse_result) {
+          return {Status::RedisParseErr, errValueNotInteger};
+        }
+
+        count_ = *parse_result;
+        i += 2;
+        continue;
+      }
+
+      if (arg == "block") {
+        if (i + 1 >= args.size()) {
+          return {Status::RedisParseErr, errInvalidSyntax};
+        }
+
+        block_ = true;
+
+        auto parse_result = ParseInt<int64_t>(args[i + 1], 10);
+        if (!parse_result) {
+          return {Status::RedisParseErr, errValueNotInteger};
+        }
+
+        if (*parse_result < 0) {
+          return {Status::RedisParseErr, errTimeoutIsNegative};
+        }
+
+        block_timeout_ = *parse_result;
+        i += 2;
+        continue;
+      }
+
+      if (arg == "noack") {
+        noack_ = true;
+      }
+
+      ++i;
+    }
+
+    if (streams_word_idx == 0) {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+
+    if ((args.size() - streams_word_idx - 1) % 2 != 0) {
+      return {Status::RedisParseErr, errUnbalancedStreamList};
+    }
+
+    size_t number_of_streams = (args.size() - streams_word_idx - 1) / 2;
+
+    for (size_t i = streams_word_idx + 1; i <= streams_word_idx + number_of_streams; ++i) {
+      streams_.push_back(args[i]);
+      const auto &id_str = args[i + number_of_streams];
+      bool get_latest = id_str == ">";
+      latest_marks_.push_back(get_latest);
+      if (!get_latest) {
+        block_ = false;
+      }
+      StreamEntryID id;
+      if (!get_latest) {
+        auto s = ParseStreamEntryID(id_str, &id);
+        if (!s.IsOK()) {
+          return s;
+        }
+      }
+      ids_.push_back(id);
+    }
+
+    return Status::OK();
+  }
+
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+
+    std::vector<redis::StreamReadResult> results;
+
+    for (size_t i = 0; i < streams_.size(); ++i) {
+      redis::StreamRangeOptions options;
+      options.reverse = false;
+      options.start = ids_[i];
+      options.end = StreamEntryID{UINT64_MAX, UINT64_MAX};
+      options.with_count = with_count_;
+      options.count = count_;
+      options.exclude_start = true;
+      options.exclude_end = false;
+
+      std::vector<StreamEntry> result;
+      auto s = stream_db.RangeWithPending(streams_[i], options, &result, group_name_, consumer_name_, noack_,
+                                          latest_marks_[i]);
+      if (!s.ok() && !s.IsNotFound()) {
+        return {Status::RedisExecErr, s.ToString()};
+      }
+
+      if (result.size() > 0) {
+        results.emplace_back(streams_[i], result);
+      }
+    }
+
+    if (block_ && results.empty()) {
+      if (conn->IsInExec()) {
+        *output = redis::MultiLen(-1);
+        return Status::OK();  // No blocking in multi-exec
+      }
+
+      return BlockingRead(srv, conn, &stream_db);
+    }
+
+    if (!block_ && results.empty()) {
+      *output = redis::MultiLen(-1);
+      return Status::OK();
+    }
+
+    return SendResults(conn, output, results);
+  }
+
+  Status SendResults(Connection *conn, std::string *output, const std::vector<StreamReadResult> &results) {
+    output->append(redis::MultiLen(results.size()));
+    int id = 0;
+    for (const auto &result : results) {
+      output->append(redis::MultiLen(2));
+      output->append(redis::BulkString(result.name));
+      output->append(redis::MultiLen(result.entries.size()));
+      for (const auto &entry : result.entries) {
+        output->append(redis::MultiLen(2));
+        output->append(redis::BulkString(entry.key));
+        if (entry.values.size() == 0 && !latest_marks_[id]) {
+          output->append(conn->NilString());
+          continue;
+        }
+        output->append(conn->MultiBulkString(entry.values));
+      }
+      ++id;
+    }
+
+    return Status::OK();
+  }
+
+  Status BlockingRead(Server *srv, Connection *conn, redis::Stream *stream_db) {
+    if (!with_count_) {
+      with_count_ = true;
+      count_ = blocked_default_count_;
+    }
+
+    srv_ = srv;
+    conn_ = conn;
+
+    srv_->BlockOnStreams(streams_, ids_, conn_);
+
+    auto bev = conn->GetBufferEvent();
+    SetCB(bev);
+
+    if (block_timeout_ > 0) {
+      timer_.reset(NewTimer(bufferevent_get_base(bev)));
+      timeval tm;
+      if (block_timeout_ > 1000) {
+        tm.tv_sec = block_timeout_ / 1000;
+        tm.tv_usec = (block_timeout_ % 1000) * 1000;
+      } else {
+        tm.tv_sec = 0;
+        tm.tv_usec = block_timeout_ * 1000;
+      }
+
+      evtimer_add(timer_.get(), &tm);
+    }
+
+    return {Status::BlockingCmd};
+  }
+
+  void OnWrite(bufferevent *bev) {
+    if (timer_ != nullptr) {
+      timer_.reset();
+    }
+
+    unblockAll();
+    conn_->SetCB(bev);
+    bufferevent_enable(bev, EV_READ);
+
+    redis::Stream stream_db(srv_->storage, conn_->GetNamespace());
+
+    std::vector<StreamReadResult> results;
+
+    for (size_t i = 0; i < streams_.size(); ++i) {
+      redis::StreamRangeOptions options;
+      options.reverse = false;
+      options.start = ids_[i];
+      options.end = StreamEntryID{UINT64_MAX, UINT64_MAX};
+      options.with_count = with_count_;
+      options.count = count_;
+      options.exclude_start = true;
+      options.exclude_end = false;
+
+      std::vector<StreamEntry> result;
+      auto s = stream_db.RangeWithPending(streams_[i], options, &result, group_name_, consumer_name_, noack_,
+                                          latest_marks_[i]);
+      if (!s.ok() && !s.IsNotFound()) {
+        conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
+        return;
+      }
+
+      if (result.size() > 0) {
+        results.emplace_back(streams_[i], result);
+      }
+    }
+
+    if (results.empty()) {
+      conn_->Reply(redis::MultiLen(-1));
+    }
+
+    SendReply(results);
+  }
+
+  void SendReply(const std::vector<StreamReadResult> &results) {
+    std::string output;
+
+    output.append(redis::MultiLen(results.size()));
+
+    for (const auto &result : results) {
+      output.append(redis::MultiLen(2));
+      output.append(redis::BulkString(result.name));
+      output.append(redis::MultiLen(result.entries.size()));
+      for (const auto &entry : result.entries) {
+        output.append(redis::MultiLen(2));
+        output.append(redis::BulkString(entry.key));
+        output.append(conn_->MultiBulkString(entry.values));
+      }
+    }
+
+    conn_->Reply(output);
+  }
+
+  void OnEvent(bufferevent *bev, int16_t events) {
+    if (events & (BEV_EVENT_EOF | BEV_EVENT_ERROR)) {
+      if (timer_ != nullptr) {
+        timer_.reset();
+      }
+      unblockAll();
+    }
+    conn_->OnEvent(bev, events);
+  }
+
+  void TimerCB(int, int16_t events) {
+    conn_->Reply(conn_->NilString());
+
+    timer_.reset();
+
+    unblockAll();
+
+    auto bev = conn_->GetBufferEvent();
+    conn_->SetCB(bev);
+    bufferevent_enable(bev, EV_READ);
+  }
+
+ private:
+  std::vector<std::string> streams_;
+  std::vector<StreamEntryID> ids_;
+  std::vector<bool> latest_marks_;
+  std::string group_name_;
+  std::string consumer_name_;
+  bool noack_ = false;
+
+  Server *srv_ = nullptr;
+  Connection *conn_ = nullptr;
+  UniqueEvent timer_;
+  uint64_t count_ = 0;
+  int64_t block_timeout_ = 0;
+  int blocked_default_count_ = 1000;
+  bool with_count_ = false;
+  bool block_ = false;
+
+  void unblockAll() { srv_->UnblockOnStreams(streams_, conn_); }
+};
+
 class CommandXTrim : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
@@ -1135,9 +1686,7 @@ class CommandXSetId : public Commander {
     stream_name_ = args[1];
 
     auto s = redis::ParseStreamEntryID(args[2], &last_id_);
-    if (!s.IsOK()) {
-      return {Status::RedisParseErr, s.Msg()};
-    }
+    if (!s.IsOK()) return s;
 
     if (args.size() == 3) {
       return Status::OK();
@@ -1155,9 +1704,7 @@ class CommandXSetId : public Commander {
       } else if (util::EqualICase(args[i], "maxdeletedid") && i + 1 < args.size()) {
         StreamEntryID id;
         s = redis::ParseStreamEntryID(args[i + 1], &id);
-        if (!s.IsOK()) {
-          return {Status::RedisParseErr, s.Msg()};
-        }
+        if (!s.IsOK()) return s;
 
         max_deleted_id_ = std::make_optional<StreamEntryID>(id.ms, id.seq);
         i += 2;
@@ -1189,15 +1736,20 @@ class CommandXSetId : public Commander {
   std::optional<uint64_t> entries_added_;
 };
 
-REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandXAdd>("xadd", -5, "write", 1, 1, 1),
-                        MakeCmdAttr<CommandXDel>("xdel", -3, "write no-dbsize-check", 1, 1, 1),
-                        // MakeCmdAttr<CommandXGroup>("xgroup", -4, "write", 2, 2, 1),
-                        MakeCmdAttr<CommandXLen>("xlen", -2, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandXInfo>("xinfo", -2, "read-only", 0, 0, 0),
-                        MakeCmdAttr<CommandXRange>("xrange", -4, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandXRevRange>("xrevrange", -2, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandXRead>("xread", -4, "read-only", 0, 0, 0),
-                        MakeCmdAttr<CommandXTrim>("xtrim", -4, "write no-dbsize-check", 1, 1, 1),
-                        MakeCmdAttr<CommandXSetId>("xsetid", -3, "write", 1, 1, 1))
+REDIS_REGISTER_COMMANDS(
+    // MakeCmdAttr<CommandXAck>("xack", -4, "write no-dbsize-check", 1, 1, 1),
+    MakeCmdAttr<CommandXAdd>("xadd", -5, "write", 1, 1, 1),
+    MakeCmdAttr<CommandXDel>("xdel", -3, "write no-dbsize-check", 1, 1, 1),
+    // MakeCmdAttr<CommandXClaim>("xclaim", -6, "write", 1, 1, 1),
+    // MakeCmdAttr<CommandAutoClaim>("xautoclaim", -6, "write", 1, 1, 1),
+    // MakeCmdAttr<CommandXGroup>("xgroup", -4, "write", 2, 2, 1),
+    MakeCmdAttr<CommandXLen>("xlen", -2, "read-only", 1, 1, 1),
+    MakeCmdAttr<CommandXInfo>("xinfo", -2, "read-only", 0, 0, 0),
+    MakeCmdAttr<CommandXRange>("xrange", -4, "read-only", 1, 1, 1),
+    MakeCmdAttr<CommandXRevRange>("xrevrange", -2, "read-only", 1, 1, 1),
+    MakeCmdAttr<CommandXRead>("xread", -4, "read-only", 0, 0, 0),
+    // MakeCmdAttr<CommandXReadGroup>("xreadgroup", -7, "write", 0, 0, 0),
+    MakeCmdAttr<CommandXTrim>("xtrim", -4, "write no-dbsize-check", 1, 1, 1),
+    MakeCmdAttr<CommandXSetId>("xsetid", -3, "write", 1, 1, 1))
 
 }  // namespace redis
