@@ -136,9 +136,7 @@ class CommandZCount : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
     Status s = ParseRangeScoreSpec(args[2], args[3], &spec_);
-    if (!s.IsOK()) {
-      return {Status::RedisParseErr, s.Msg()};
-    }
+    if (!s.IsOK()) return s;
     return Commander::Parse(args);
   }
 
@@ -204,9 +202,7 @@ class CommandZLexCount : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
     Status s = ParseRangeLexSpec(args[2], args[3], &spec_);
-    if (!s.IsOK()) {
-      return {Status::RedisParseErr, s.Msg()};
-    }
+    if (!s.IsOK()) return s;
 
     return Commander::Parse(args);
   }
@@ -369,7 +365,7 @@ class CommandBZPop : public BlockingCommander {
     redis::ZSet zset_db(srv_->storage, conn_->GetNamespace());
     auto s = PopFromMultipleZsets(&zset_db, keys_, min_, 1, &user_key, &member_scores);
     if (!s.ok()) {
-      conn_->Reply(redis::Error("ERR " + s.ToString()));
+      conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
       return true;
     }
 
@@ -548,7 +544,7 @@ class CommandBZMPop : public BlockingCommander {
     redis::ZSet zset_db(srv_->storage, conn_->GetNamespace());
     auto s = PopFromMultipleZsets(&zset_db, keys_, flag_ == ZSET_MIN, count_, &user_key, &member_scores);
     if (!s.ok()) {
-      conn_->Reply(redis::Error("ERR " + s.ToString()));
+      conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
       return true;
     }
 
@@ -985,9 +981,7 @@ class CommandZRemRangeByScore : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
     Status s = ParseRangeScoreSpec(args[2], args[3], &spec_);
-    if (!s.IsOK()) {
-      return {Status::RedisParseErr, s.Msg()};
-    }
+    if (!s.IsOK()) return s;
     return Commander::Parse(args);
   }
 
@@ -1014,9 +1008,7 @@ class CommandZRemRangeByLex : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
     Status s = ParseRangeLexSpec(args[2], args[3], &spec_);
-    if (!s.IsOK()) {
-      return {Status::RedisParseErr, s.Msg()};
-    }
+    if (!s.IsOK()) return s;
     return Commander::Parse(args);
   }
 
