@@ -106,13 +106,20 @@ struct CommandKeyRange {
   // step length of key position
   // e.g. key step 2 means "key other key other ..." sequence
   int key_step;
+
+  template <typename F>
+  void ForEachKey(F &&f, const std::vector<std::string> &args) const {
+    for (size_t i = first_key; last_key > 0 ? i <= size_t(last_key) : i <= args.size() + last_key; i += key_step) {
+      std::forward<F>(f)(args[i]);
+    }
+  }
 };
 
 using CommandKeyRangeGen = std::function<CommandKeyRange(const std::vector<std::string> &)>;
 
 using CommandKeyRangeVecGen = std::function<std::vector<CommandKeyRange>(const std::vector<std::string> &)>;
 
-using AdditionalFlagGen = std::function<uint64_t(const std::vector<std::string> &)>;
+using AdditionalFlagGen = std::function<uint64_t(uint64_t, const std::vector<std::string> &)>;
 
 struct CommandAttributes {
   // command name
@@ -146,13 +153,34 @@ struct CommandAttributes {
 
   auto GenerateFlags(const std::vector<std::string> &args) const {
     uint64_t res = flags;
-    if (flag_gen) res |= flag_gen(args);
+    if (flag_gen) res = flag_gen(res, args);
     return res;
   }
 
   bool CheckArity(int cmd_size) const {
     return !((arity > 0 && cmd_size != arity) || (arity < 0 && cmd_size < -arity));
   }
+
+  template <typename F>
+  void ForEachKeyRange(F &&f, const std::vector<std::string> &args) const {
+    if (key_range.first_key > 0) {
+      std::forward<F>(f)(args, key_range);
+    } else if (key_range.first_key == -1) {
+      redis::CommandKeyRange range = key_range_gen(args);
+
+      if (range.first_key > 0) {
+        std::forward<F>(f)(args, range);
+      }
+    } else if (key_range.first_key == -2) {
+      std::vector<redis::CommandKeyRange> vec_range = key_range_vec_gen(args);
+
+      for (const auto &range : vec_range) {
+        if (range.first_key > 0) {
+          std::forward<F>(f)(args, range);
+        }
+      }
+    }
+  }
 };
 
 using CommandMap = std::map<std::string, const CommandAttributes *>;
