@@ -26,7 +26,10 @@
 #include <shared_mutex>
 
 #include "commands/commander.h"
+#include "commands/error_constants.h"
 #include "fmt/format.h"
+#include "search/indexer.h"
+#include "server/redis_reply.h"
 #include "string_util.h"
 #ifdef ENABLE_OPENSSL
 #include <event2/bufferevent_ssl.h>
@@ -84,7 +87,7 @@ void Connection::OnRead(struct bufferevent *bev) {
   auto s = req_.Tokenize(Input());
   if (!s.IsOK()) {
     EnableFlag(redis::Connection::kCloseAfterReply);
-    Reply(redis::Error("ERR " + s.Msg()));
+    Reply(redis::Error(s));
     LOG(INFO) << "[connection] Failed to tokenize the request. Error: " << s.Msg();
     return;
   }
@@ -138,7 +141,7 @@ std::string Connection::Bool(bool b) const {
 }
 
 std::string Connection::MultiBulkString(const std::vector<std::string> &values) const {
-  std::string result = "*" + std::to_string(values.size()) + CRLF;
+  std::string result = MultiLen(values.size());
   for (const auto &value : values) {
     if (value.empty()) {
       result += NilString();
@@ -151,7 +154,7 @@ std::string Connection::MultiBulkString(const std::vector<std::string> &values)
 
 std::string Connection::MultiBulkString(const std::vector<std::string> &values,
                                         const std::vector<rocksdb::Status> &statuses) const {
-  std::string result = "*" + std::to_string(values.size()) + CRLF;
+  std::string result = MultiLen(values.size());
   for (size_t i = 0; i < values.size(); i++) {
     if (i < statuses.size() && !statuses[i].ok()) {
       result += NilString();
@@ -220,6 +223,7 @@ std::string Connection::GetFlags() const {
   if (IsFlagEnabled(kSlave)) flags.append("S");
   if (IsFlagEnabled(kCloseAfterReply)) flags.append("c");
   if (IsFlagEnabled(kMonitor)) flags.append("M");
+  if (IsFlagEnabled(kAsking)) flags.append("A");
   if (!subscribe_channels_.empty() || !subscribe_patterns_.empty()) flags.append("P");
   if (flags.empty()) flags = "N";
   return flags;
@@ -408,34 +412,39 @@ Status Connection::ExecuteCommand(const std::string &cmd_name, const std::vector
   return s;
 }
 
+static bool IsHashOrJsonCommand(const std::string &cmd) {
+  return util::HasPrefix(cmd, "h") || util::HasPrefix(cmd, "json.");
+}
+
 void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
-  Config *config = srv_->GetConfig();
-  std::string reply, password = config->requirepass;
+  const Config *config = srv_->GetConfig();
+  std::string reply;
+  std::string password = config->requirepass;
 
   while (!to_process_cmds->empty()) {
-    auto cmd_tokens = to_process_cmds->front();
+    CommandTokens cmd_tokens = std::move(to_process_cmds->front());
     to_process_cmds->pop_front();
     if (cmd_tokens.empty()) continue;
 
     bool is_multi_exec = IsFlagEnabled(Connection::kMultiExec);
     if (IsFlagEnabled(redis::Connection::kCloseAfterReply) && !is_multi_exec) break;
 
-    auto cmd_s = srv_->LookupAndCreateCommand(cmd_tokens.front());
+    auto cmd_s = Server::LookupAndCreateCommand(cmd_tokens.front());
     if (!cmd_s.IsOK()) {
       if (is_multi_exec) multi_error_ = true;
-      Reply(redis::Error("ERR unknown command " + cmd_tokens.front()));
+      Reply(redis::Error({Status::NotOK, "unknown command " + cmd_tokens.front()}));
       continue;
     }
     auto current_cmd = std::move(*cmd_s);
 
-    const auto attributes = current_cmd->GetAttributes();
+    const auto &attributes = current_cmd->GetAttributes();
     auto cmd_name = attributes->name;
     auto cmd_flags = attributes->GenerateFlags(cmd_tokens);
 
     if (GetNamespace().empty()) {
       if (!password.empty()) {
         if (cmd_name != "auth" && cmd_name != "hello") {
-          Reply(redis::Error("NOAUTH Authentication required."));
+          Reply(redis::Error({Status::RedisNoAuth, "Authentication required."}));
           continue;
         }
       } else {
@@ -468,7 +477,7 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
     }
 
     if (srv_->IsLoading() && !(cmd_flags & kCmdLoading)) {
-      Reply(redis::Error("LOADING kvrocks is restoring the db from backup"));
+      Reply(redis::Error({Status::RedisLoading, errRestoringBackup}));
       if (is_multi_exec) multi_error_ = true;
       continue;
     }
@@ -476,7 +485,7 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
     int tokens = static_cast<int>(cmd_tokens.size());
     if (!attributes->CheckArity(tokens)) {
       if (is_multi_exec) multi_error_ = true;
-      Reply(redis::Error("ERR wrong number of arguments"));
+      Reply(redis::Error({Status::NotOK, "wrong number of arguments"}));
       continue;
     }
 
@@ -484,12 +493,12 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
     auto s = current_cmd->Parse();
     if (!s.IsOK()) {
       if (is_multi_exec) multi_error_ = true;
-      Reply(redis::Error("ERR " + s.Msg()));
+      Reply(redis::Error(s));
       continue;
     }
 
     if (is_multi_exec && (cmd_flags & kCmdNoMulti)) {
-      Reply(redis::Error("ERR Can't execute " + cmd_name + " in MULTI"));
+      Reply(redis::Error({Status::NotOK, "Can't execute " + cmd_name + " in MULTI"}));
       multi_error_ = true;
       continue;
     }
@@ -498,11 +507,16 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
       s = srv_->cluster->CanExecByMySelf(attributes, cmd_tokens, this);
       if (!s.IsOK()) {
         if (is_multi_exec) multi_error_ = true;
-        Reply(redis::Error(s.Msg()));
+        Reply(redis::Error(s));
         continue;
       }
     }
 
+    // reset the ASKING flag after executing the next query
+    if (IsFlagEnabled(kAsking)) {
+      DisableFlag(kAsking);
+    }
+
     // We don't execute commands, but queue them, ant then execute in EXEC command
     if (is_multi_exec && !in_exec_ && !(cmd_flags & kCmdMulti)) {
       multi_cmds_.emplace_back(cmd_tokens);
@@ -511,26 +525,53 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
     }
 
     if (config->slave_readonly && srv_->IsSlave() && (cmd_flags & kCmdWrite)) {
-      Reply(redis::Error("READONLY You can't write against a read only slave."));
+      Reply(redis::Error({Status::RedisReadOnly, "You can't write against a read only slave."}));
       continue;
     }
 
     if ((cmd_flags & kCmdWrite) && !(cmd_flags & kCmdNoDBSizeCheck) && srv_->storage->ReachedDBSizeLimit()) {
-      Reply(redis::Error("ERR write command not allowed when reached max-db-size."));
+      Reply(redis::Error({Status::NotOK, "write command not allowed when reached max-db-size."}));
       continue;
     }
 
     if (!config->slave_serve_stale_data && srv_->IsSlave() && cmd_name != "info" && cmd_name != "slaveof" &&
         srv_->GetReplicationState() != kReplConnected) {
-      Reply(
-          redis::Error("MASTERDOWN Link with MASTER is down "
-                       "and slave-serve-stale-data is set to 'no'."));
+      Reply(redis::Error({Status::RedisMasterDown,
+                          "Link with MASTER is down "
+                          "and slave-serve-stale-data is set to 'no'."}));
       continue;
     }
 
+    // TODO: transaction support for index recording
+    std::vector<GlobalIndexer::RecordResult> index_records;
+    if (IsHashOrJsonCommand(cmd_name) && (attributes->flags & redis::kCmdWrite) && !config->cluster_enabled) {
+      attributes->ForEachKeyRange(
+          [&, this](const std::vector<std::string> &args, const CommandKeyRange &key_range) {
+            key_range.ForEachKey(
+                [&, this](const std::string &key) {
+                  auto res = srv_->indexer.Record(key, ns_);
+                  if (res.IsOK()) {
+                    index_records.push_back(*res);
+                  } else if (!res.Is<Status::NoPrefixMatched>() && !res.Is<Status::TypeMismatched>()) {
+                    LOG(WARNING) << "index recording failed for key: " << key;
+                  }
+                },
+                args);
+          },
+          cmd_tokens);
+    }
+
     SetLastCmd(cmd_name);
     s = ExecuteCommand(cmd_name, cmd_tokens, current_cmd.get(), &reply);
 
+    // TODO: transaction support for index updating
+    for (const auto &record : index_records) {
+      auto s = GlobalIndexer::Update(record);
+      if (!s.IsOK() && !s.Is<Status::TypeMismatched>()) {
+        LOG(WARNING) << "index updating failed for key: " << record.key;
+      }
+    }
+
     // Break the execution loop when occurring the blocking command like BLPOP or BRPOP,
     // it will suspend the connection and wait for the wakeup signal.
     if (s.Is<Status::BlockingCmd>()) {
@@ -544,7 +585,7 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
 
     // Reply for MULTI
     if (!s.IsOK()) {
-      Reply(redis::Error("ERR " + s.Msg()));
+      Reply(redis::Error(s));
       continue;
     }
 
