@@ -45,6 +45,8 @@ class Connection : public EvbufCallbackBase<Connection> {
     kCloseAfterReply = 1 << 6,
     kCloseAsync = 1 << 7,
     kMultiExec = 1 << 8,
+    kReadOnly = 1 << 9,
+    kAsking = 1 << 10,
   };
 
   explicit Connection(bufferevent *bev, Worker *owner);
@@ -101,6 +103,9 @@ class Connection : public EvbufCallbackBase<Connection> {
   std::string HeaderOfAttribute(T len) const {
     return "|" + std::to_string(len) + CRLF;
   }
+  std::string HeaderOfPush(int64_t len) const {
+    return protocol_version_ == RESP::v3 ? ">" + std::to_string(len) + CRLF : MultiLen(len);
+  }
 
   using UnsubscribeCallback = std::function<void(std::string, int)>;
   void SubscribeChannel(const std::string &channel);
