@@ -36,10 +36,6 @@
 
 namespace redis {
 
-const size_t PROTO_INLINE_MAX_SIZE = 16 * 1024L;
-const size_t PROTO_BULK_MAX_SIZE = 512 * 1024L * 1024L;
-const size_t PROTO_MULTI_MAX_SIZE = 1024 * 1024L;
-
 Status Request::Tokenize(evbuffer *input) {
   size_t pipeline_size = 0;
 
