@@ -35,12 +35,18 @@
 #include "storage/redis_metadata.h"
 #include "storage/storage.h"
 #include "time_util.h"
+#include "types/redis_hash.h"
+#include "types/redis_list.h"
+#include "types/redis_set.h"
+#include "types/redis_string.h"
+#include "types/redis_zset.h"
 
 namespace redis {
 
-Database::Database(engine::Storage *storage, std::string ns) : storage_(storage), namespace_(std::move(ns)) {
-  metadata_cf_handle_ = storage->GetCFHandle("metadata");
-}
+Database::Database(engine::Storage *storage, std::string ns)
+    : storage_(storage),
+      metadata_cf_handle_(storage->GetCFHandle(ColumnFamilyID::Metadata)),
+      namespace_(std::move(ns)) {}
 
 // Some data types may support reading multiple types of metadata.
 // For example, bitmap supports reading string metadata and bitmap metadata.
@@ -84,25 +90,25 @@ rocksdb::Status Database::ParseMetadata(RedisTypes types, Slice *bytes, Metadata
   return s;
 }
 
-rocksdb::Status Database::GetMetadata(RedisTypes types, const Slice &ns_key, Metadata *metadata) {
+rocksdb::Status Database::GetMetadata(GetOptions options, RedisTypes types, const Slice &ns_key, Metadata *metadata) {
   std::string raw_value;
   Slice rest;
-  return GetMetadata(types, ns_key, &raw_value, metadata, &rest);
+  return GetMetadata(options, types, ns_key, &raw_value, metadata, &rest);
 }
 
-rocksdb::Status Database::GetMetadata(RedisTypes types, const Slice &ns_key, std::string *raw_value, Metadata *metadata,
-                                      Slice *rest) {
-  auto s = GetRawMetadata(ns_key, raw_value);
+rocksdb::Status Database::GetMetadata(GetOptions options, RedisTypes types, const Slice &ns_key, std::string *raw_value,
+                                      Metadata *metadata, Slice *rest) {
+  auto s = GetRawMetadata(options, ns_key, raw_value);
   *rest = *raw_value;
   if (!s.ok()) return s;
   return ParseMetadata(types, rest, metadata);
 }
 
-rocksdb::Status Database::GetRawMetadata(const Slice &ns_key, std::string *bytes) {
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
-  return storage_->Get(read_options, metadata_cf_handle_, ns_key, bytes);
+rocksdb::Status Database::GetRawMetadata(GetOptions options, const Slice &ns_key, std::string *bytes) {
+  rocksdb::ReadOptions opts;
+  // If options.snapshot == nullptr, we can avoid allocating a snapshot here.
+  opts.snapshot = options.snapshot;
+  return storage_->Get(opts, metadata_cf_handle_, ns_key, bytes);
 }
 
 rocksdb::Status Database::Expire(const Slice &user_key, uint64_t timestamp) {
@@ -204,25 +210,12 @@ rocksdb::Status Database::MDel(const std::vector<Slice> &keys, uint64_t *deleted
 }
 
 rocksdb::Status Database::Exists(const std::vector<Slice> &keys, int *ret) {
-  *ret = 0;
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
-
-  rocksdb::Status s;
-  std::string value;
+  std::vector<std::string> ns_keys;
+  ns_keys.reserve(keys.size());
   for (const auto &key : keys) {
-    std::string ns_key = AppendNamespacePrefix(key);
-    s = storage_->Get(read_options, metadata_cf_handle_, ns_key, &value);
-    if (!s.ok() && !s.IsNotFound()) return s;
-    if (s.ok()) {
-      Metadata metadata(kRedisNone, false);
-      s = metadata.Decode(value);
-      if (!s.ok()) return s;
-      if (!metadata.Expired()) *ret += 1;
-    }
+    ns_keys.emplace_back(AppendNamespacePrefix(key));
   }
-  return rocksdb::Status::OK();
+  return existsInternal(ns_keys, ret);
 }
 
 rocksdb::Status Database::TTL(const Slice &user_key, int64_t *ttl) {
@@ -247,7 +240,7 @@ rocksdb::Status Database::TTL(const Slice &user_key, int64_t *ttl) {
 rocksdb::Status Database::GetExpireTime(const Slice &user_key, uint64_t *timestamp) {
   std::string ns_key = AppendNamespacePrefix(user_key);
   Metadata metadata(kRedisNone, false);
-  auto s = GetMetadata(RedisTypes::All(), ns_key, &metadata);
+  auto s = GetMetadata(GetOptions{}, RedisTypes::All(), ns_key, &metadata);
   if (!s.ok()) return s;
   *timestamp = metadata.expire;
 
@@ -323,7 +316,7 @@ rocksdb::Status Database::Keys(const std::string &prefix, std::vector<std::strin
 }
 
 rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const std::string &prefix,
-                               std::vector<std::string> *keys, std::string *end_cursor) {
+                               std::vector<std::string> *keys, std::string *end_cursor, RedisType type) {
   end_cursor->clear();
   uint64_t cnt = 0;
   uint16_t slot_start = 0;
@@ -368,6 +361,8 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
       auto s = metadata.Decode(iter->value());
       if (!s.ok()) continue;
 
+      if (type != kRedisNone && type != metadata.Type()) continue;
+
       if (metadata.Expired()) continue;
       std::tie(std::ignore, user_key) = ExtractNamespaceKey<std::string>(iter->key(), storage_->IsSlotIdEncoded());
       keys->emplace_back(user_key);
@@ -438,18 +433,10 @@ rocksdb::Status Database::RandomKey(const std::string &cursor, std::string *key)
 }
 
 rocksdb::Status Database::FlushDB() {
-  std::string begin_key, end_key;
-  std::string prefix = ComposeNamespaceKey(namespace_, "", false);
-  auto s = FindKeyRangeWithPrefix(prefix, std::string(), &begin_key, &end_key);
-  if (!s.ok()) {
-    return rocksdb::Status::OK();
-  }
-  s = storage_->DeleteRange(begin_key, end_key);
-  if (!s.ok()) {
-    return s;
-  }
+  auto begin_key = ComposeNamespaceKey(namespace_, "", false);
+  auto end_key = util::StringNext(begin_key);
 
-  return rocksdb::Status::OK();
+  return storage_->DeleteRange(begin_key, end_key);
 }
 
 rocksdb::Status Database::FlushAll() {
@@ -466,12 +453,8 @@ rocksdb::Status Database::FlushAll() {
   if (!iter->Valid()) {
     return rocksdb::Status::OK();
   }
-  auto last_key = iter->key().ToString();
-  auto s = storage_->DeleteRange(first_key, last_key);
-  if (!s.ok()) {
-    return s;
-  }
-  return rocksdb::Status::OK();
+  auto last_key = util::StringNext(iter->key().ToString());
+  return storage_->DeleteRange(first_key, last_key);
 }
 
 rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *infos) {
@@ -517,7 +500,7 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
 
   if (metadata.Type() == kRedisList) {
     ListMetadata list_metadata(false);
-    s = GetMetadata({kRedisList}, ns_key, &list_metadata);
+    s = GetMetadata(GetOptions{}, {kRedisList}, ns_key, &list_metadata);
     if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
     infos->emplace_back("head");
     infos->emplace_back(std::to_string(list_metadata.head));
@@ -528,80 +511,15 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::Type(const Slice &user_key, RedisType *type) {
-  std::string ns_key = AppendNamespacePrefix(user_key);
-
-  *type = kRedisNone;
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
-  std::string value;
-  rocksdb::Status s = storage_->Get(read_options, metadata_cf_handle_, ns_key, &value);
-  if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
-
-  Metadata metadata(kRedisNone, false);
-  s = metadata.Decode(value);
-  if (!s.ok()) return s;
-  if (metadata.Expired()) {
-    *type = kRedisNone;
-  } else {
-    *type = metadata.Type();
-  }
-  return rocksdb::Status::OK();
+rocksdb::Status Database::Type(const Slice &key, RedisType *type) {
+  std::string ns_key = AppendNamespacePrefix(key);
+  return typeInternal(ns_key, type);
 }
 
 std::string Database::AppendNamespacePrefix(const Slice &user_key) {
   return ComposeNamespaceKey(namespace_, user_key, storage_->IsSlotIdEncoded());
 }
 
-rocksdb::Status Database::FindKeyRangeWithPrefix(const std::string &prefix, const std::string &prefix_end,
-                                                 std::string *begin, std::string *end,
-                                                 rocksdb::ColumnFamilyHandle *cf_handle) {
-  if (cf_handle == nullptr) {
-    cf_handle = metadata_cf_handle_;
-  }
-  if (prefix.empty()) {
-    return rocksdb::Status::NotFound();
-  }
-  begin->clear();
-  end->clear();
-
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
-  auto iter = util::UniqueIterator(storage_, read_options, cf_handle);
-  iter->Seek(prefix);
-  if (!iter->Valid() || !iter->key().starts_with(prefix)) {
-    return rocksdb::Status::NotFound();
-  }
-  *begin = iter->key().ToString();
-
-  // it's ok to increase the last char in prefix as the boundary of the prefix
-  // while we limit the namespace last char shouldn't be larger than 128.
-  std::string next_prefix;
-  if (!prefix_end.empty()) {
-    next_prefix = prefix_end;
-  } else {
-    next_prefix = prefix;
-    char last_char = next_prefix.back();
-    last_char++;
-    next_prefix.pop_back();
-    next_prefix.push_back(last_char);
-  }
-  iter->SeekForPrev(next_prefix);
-  int max_prev_limit = 128;  // prevent unpredicted long while loop
-  int i = 0;
-  // reversed seek the key til with prefix or end of the iterator
-  while (i++ < max_prev_limit && iter->Valid() && !iter->key().starts_with(prefix)) {
-    iter->Prev();
-  }
-  if (!iter->Valid() || !iter->key().starts_with(prefix)) {
-    return rocksdb::Status::NotFound();
-  }
-  *end = iter->key().ToString();
-  return rocksdb::Status::OK();
-}
-
 rocksdb::Status Database::ClearKeysOfSlot(const rocksdb::Slice &ns, int slot) {
   if (!storage_->IsSlotIdEncoded()) {
     return rocksdb::Status::Aborted("It is not in cluster mode");
@@ -618,8 +536,7 @@ rocksdb::Status Database::ClearKeysOfSlot(const rocksdb::Slice &ns, int slot) {
 
 rocksdb::Status Database::KeyExist(const std::string &key) {
   int cnt = 0;
-  std::vector<rocksdb::Slice> keys;
-  keys.emplace_back(key);
+  std::vector<rocksdb::Slice> keys{key};
   auto s = Exists(keys, &cnt);
   if (!s.ok()) {
     return s;
@@ -636,12 +553,11 @@ rocksdb::Status SubKeyScanner::Scan(RedisType type, const Slice &user_key, const
   uint64_t cnt = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
   Metadata metadata(type, false);
-  rocksdb::Status s = GetMetadata({type}, ns_key, &metadata);
+  LatestSnapShot ss(storage_);
+  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, {type}, ns_key, &metadata);
   if (!s.ok()) return s;
 
-  LatestSnapShot ss(storage_);
   rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
   auto iter = util::UniqueIterator(storage_, read_options);
   std::string match_prefix_key =
       InternalKey(ns_key, subkey_prefix, metadata.version, storage_->IsSlotIdEncoded()).Encode();
@@ -699,28 +615,71 @@ Status WriteBatchLogData::Decode(const rocksdb::Slice &blob) {
   return Status::OK();
 }
 
-rocksdb::Status Database::Rename(const std::string &key, const std::string &new_key, bool nx, bool *ret) {
-  *ret = true;
-  std::string ns_key = AppendNamespacePrefix(key);
-  std::string new_ns_key = AppendNamespacePrefix(new_key);
+rocksdb::Status Database::existsInternal(const std::vector<std::string> &keys, int *ret) {
+  *ret = 0;
+  LatestSnapShot ss(storage_);
+  rocksdb::ReadOptions read_options;
+  read_options.snapshot = ss.GetSnapShot();
 
-  std::vector<std::string> lock_keys = {ns_key, new_ns_key};
+  rocksdb::Status s;
+  std::string value;
+  for (const auto &key : keys) {
+    s = storage_->Get(read_options, metadata_cf_handle_, key, &value);
+    if (!s.ok() && !s.IsNotFound()) return s;
+    if (s.ok()) {
+      Metadata metadata(kRedisNone, false);
+      s = metadata.Decode(value);
+      if (!s.ok()) return s;
+      if (!metadata.Expired()) *ret += 1;
+    }
+  }
+  return rocksdb::Status::OK();
+}
+
+rocksdb::Status Database::typeInternal(const Slice &key, RedisType *type) {
+  *type = kRedisNone;
+  LatestSnapShot ss(storage_);
+  rocksdb::ReadOptions read_options;
+  read_options.snapshot = ss.GetSnapShot();
+  std::string value;
+  rocksdb::Status s = storage_->Get(read_options, metadata_cf_handle_, key, &value);
+  if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
+
+  Metadata metadata(kRedisNone, false);
+  s = metadata.Decode(value);
+  if (!s.ok()) return s;
+  if (metadata.Expired()) {
+    *type = kRedisNone;
+  } else {
+    *type = metadata.Type();
+  }
+  return rocksdb::Status::OK();
+}
+
+rocksdb::Status Database::Copy(const std::string &key, const std::string &new_key, bool nx, bool delete_old,
+                               CopyResult *res) {
+  std::vector<std::string> lock_keys = {key, new_key};
   MultiLockGuard guard(storage_->GetLockManager(), lock_keys);
 
   RedisType type = kRedisNone;
-  auto s = Type(key, &type);
+  auto s = typeInternal(key, &type);
   if (!s.ok()) return s;
-  if (type == kRedisNone) return rocksdb::Status::InvalidArgument("ERR no such key");
+  if (type == kRedisNone) {
+    *res = CopyResult::KEY_NOT_EXIST;
+    return rocksdb::Status::OK();
+  }
 
   if (nx) {
     int exist = 0;
-    if (s = Exists({new_key}, &exist), !s.ok()) return s;
+    if (s = existsInternal({new_key}, &exist), !s.ok()) return s;
     if (exist > 0) {
-      *ret = false;
+      *res = CopyResult::KEY_ALREADY_EXIST;
       return rocksdb::Status::OK();
     }
   }
 
+  *res = CopyResult::DONE;
+
   if (key == new_key) return rocksdb::Status::OK();
 
   auto batch = storage_->GetWriteBatchBase();
@@ -728,21 +687,23 @@ rocksdb::Status Database::Rename(const std::string &key, const std::string &new_
   batch->PutLogData(log_data.Encode());
 
   engine::DBIterator iter(storage_, rocksdb::ReadOptions());
-  iter.Seek(ns_key);
+  iter.Seek(key);
 
+  if (delete_old) {
+    batch->Delete(metadata_cf_handle_, key);
+  }
   // copy metadata
-  batch->Delete(metadata_cf_handle_, ns_key);
-  batch->Put(metadata_cf_handle_, new_ns_key, iter.Value());
+  batch->Put(metadata_cf_handle_, new_key, iter.Value());
 
   auto subkey_iter = iter.GetSubKeyIterator();
 
   if (subkey_iter != nullptr) {
-    auto zset_score_cf = type == kRedisZSet ? storage_->GetCFHandle(engine::kZSetScoreColumnFamilyName) : nullptr;
+    auto zset_score_cf = type == kRedisZSet ? storage_->GetCFHandle(ColumnFamilyID::SecondarySubkey) : nullptr;
 
     for (subkey_iter->Seek(); subkey_iter->Valid(); subkey_iter->Next()) {
       InternalKey from_ikey(subkey_iter->Key(), storage_->IsSlotIdEncoded());
       std::string to_ikey =
-          InternalKey(new_ns_key, from_ikey.GetSubKey(), from_ikey.GetVersion(), storage_->IsSlotIdEncoded()).Encode();
+          InternalKey(new_key, from_ikey.GetSubKey(), from_ikey.GetVersion(), storage_->IsSlotIdEncoded()).Encode();
       // copy sub key
       batch->Put(subkey_iter->ColumnFamilyHandle(), to_ikey, subkey_iter->Value());
 
@@ -753,7 +714,7 @@ rocksdb::Status Database::Rename(const std::string &key, const std::string &new_
         score_bytes.append(from_ikey.GetSubKey().ToString());
         // copy score key
         std::string score_key =
-            InternalKey(new_ns_key, score_bytes, from_ikey.GetVersion(), storage_->IsSlotIdEncoded()).Encode();
+            InternalKey(new_key, score_bytes, from_ikey.GetVersion(), storage_->IsSlotIdEncoded()).Encode();
         batch->Put(zset_score_cf, score_key, Slice());
       }
     }
@@ -761,4 +722,212 @@ rocksdb::Status Database::Rename(const std::string &key, const std::string &new_
 
   return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
+
+std::optional<std::string> Database::lookupKeyByPattern(const std::string &pattern, const std::string &subst) {
+  if (pattern == "#") {
+    return subst;
+  }
+
+  auto match_pos = pattern.find('*');
+  if (match_pos == std::string::npos) {
+    return std::nullopt;
+  }
+
+  // hash field
+  std::string field;
+  auto arrow_pos = pattern.find("->", match_pos + 1);
+  if (arrow_pos != std::string::npos && arrow_pos + 2 < pattern.size()) {
+    field = pattern.substr(arrow_pos + 2);
+  }
+
+  std::string key = pattern.substr(0, match_pos + 1);
+  key.replace(match_pos, 1, subst);
+
+  std::string value;
+  RedisType type = RedisType::kRedisNone;
+  if (!field.empty()) {
+    auto hash_db = redis::Hash(storage_, namespace_);
+    if (auto s = hash_db.Type(key, &type); !s.ok() || type != RedisType::kRedisHash) {
+      return std::nullopt;
+    }
+
+    if (auto s = hash_db.Get(key, field, &value); !s.ok()) {
+      return std::nullopt;
+    }
+  } else {
+    auto string_db = redis::String(storage_, namespace_);
+    if (auto s = string_db.Type(key, &type); !s.ok() || type != RedisType::kRedisString) {
+      return std::nullopt;
+    }
+    if (auto s = string_db.Get(key, &value); !s.ok()) {
+      return std::nullopt;
+    }
+  }
+  return value;
+}
+
+rocksdb::Status Database::Sort(RedisType type, const std::string &key, const SortArgument &args,
+                               std::vector<std::optional<std::string>> *elems, SortResult *res) {
+  // Obtain the length of the object to sort.
+  const std::string ns_key = AppendNamespacePrefix(key);
+  Metadata metadata(type, false);
+  auto s = GetMetadata(GetOptions{}, {type}, ns_key, &metadata);
+  if (!s.ok()) return s;
+
+  if (metadata.size > SORT_LENGTH_LIMIT) {
+    *res = SortResult::LIMIT_EXCEEDED;
+    return rocksdb::Status::OK();
+  }
+  auto vectorlen = static_cast<int>(metadata.size);
+
+  // Adjust the offset and count of the limit
+  int offset = args.offset >= vectorlen ? 0 : std::clamp(args.offset, 0, vectorlen - 1);
+  int count = args.offset >= vectorlen ? 0 : std::clamp(args.count, -1, vectorlen - offset);
+  if (count == -1) count = vectorlen - offset;
+
+  // Get the elements that need to be sorted
+  std::vector<std::string> str_vec;
+  if (count != 0) {
+    if (type == RedisType::kRedisList) {
+      auto list_db = redis::List(storage_, namespace_);
+
+      if (args.dontsort) {
+        if (args.desc) {
+          s = list_db.Range(key, -count - offset, -1 - offset, &str_vec);
+          if (!s.ok()) return s;
+          std::reverse(str_vec.begin(), str_vec.end());
+        } else {
+          s = list_db.Range(key, offset, offset + count - 1, &str_vec);
+          if (!s.ok()) return s;
+        }
+      } else {
+        s = list_db.Range(key, 0, -1, &str_vec);
+        if (!s.ok()) return s;
+      }
+    } else if (type == RedisType::kRedisSet) {
+      auto set_db = redis::Set(storage_, namespace_);
+      s = set_db.Members(key, &str_vec);
+      if (!s.ok()) return s;
+
+      if (args.dontsort) {
+        str_vec = std::vector(std::make_move_iterator(str_vec.begin() + offset),
+                              std::make_move_iterator(str_vec.begin() + offset + count));
+      }
+    } else if (type == RedisType::kRedisZSet) {
+      auto zset_db = redis::ZSet(storage_, namespace_);
+      std::vector<MemberScore> member_scores;
+
+      if (args.dontsort) {
+        RangeRankSpec spec;
+        spec.start = offset;
+        spec.stop = offset + count - 1;
+        spec.reversed = args.desc;
+        s = zset_db.RangeByRank(key, spec, &member_scores, nullptr);
+        if (!s.ok()) return s;
+
+        for (auto &member_score : member_scores) {
+          str_vec.emplace_back(std::move(member_score.member));
+        }
+      } else {
+        s = zset_db.GetAllMemberScores(key, &member_scores);
+        if (!s.ok()) return s;
+
+        for (auto &member_score : member_scores) {
+          str_vec.emplace_back(std::move(member_score.member));
+        }
+      }
+    } else {
+      *res = SortResult::UNKNOWN_TYPE;
+      return s;
+    }
+  }
+
+  std::vector<RedisSortObject> sort_vec(str_vec.size());
+  for (size_t i = 0; i < str_vec.size(); ++i) {
+    sort_vec[i].obj = str_vec[i];
+  }
+
+  // Sort by BY, ALPHA, ASC/DESC
+  if (!args.dontsort) {
+    for (size_t i = 0; i < sort_vec.size(); ++i) {
+      std::string byval;
+      if (!args.sortby.empty()) {
+        auto lookup = lookupKeyByPattern(args.sortby, str_vec[i]);
+        if (!lookup.has_value()) continue;
+        byval = std::move(lookup.value());
+      } else {
+        byval = str_vec[i];
+      }
+
+      if (args.alpha && !args.sortby.empty()) {
+        sort_vec[i].v = byval;
+      } else if (!args.alpha && !byval.empty()) {
+        auto double_byval = ParseFloat<double>(byval);
+        if (!double_byval) {
+          *res = SortResult::DOUBLE_CONVERT_ERROR;
+          return rocksdb::Status::OK();
+        }
+        sort_vec[i].v = *double_byval;
+      }
+    }
+
+    std::sort(sort_vec.begin(), sort_vec.end(), [&args](const RedisSortObject &a, const RedisSortObject &b) {
+      return RedisSortObject::SortCompare(a, b, args);
+    });
+
+    // Gets the element specified by Limit
+    if (offset != 0 || count != vectorlen) {
+      sort_vec = std::vector(std::make_move_iterator(sort_vec.begin() + offset),
+                             std::make_move_iterator(sort_vec.begin() + offset + count));
+    }
+  }
+
+  // Perform storage
+  for (auto &elem : sort_vec) {
+    if (args.getpatterns.empty()) {
+      elems->emplace_back(elem.obj);
+    }
+    for (const std::string &pattern : args.getpatterns) {
+      std::optional<std::string> val = lookupKeyByPattern(pattern, elem.obj);
+      if (val.has_value()) {
+        elems->emplace_back(val.value());
+      } else {
+        elems->emplace_back(std::nullopt);
+      }
+    }
+  }
+
+  if (!args.storekey.empty()) {
+    std::vector<std::string> store_elems;
+    store_elems.reserve(elems->size());
+    for (const auto &e : *elems) {
+      store_elems.emplace_back(e.value_or(""));
+    }
+    redis::List list_db(storage_, namespace_);
+    s = list_db.Trim(args.storekey, -1, 0);
+    if (!s.ok()) return s;
+    uint64_t new_size = 0;
+    s = list_db.Push(args.storekey, std::vector<Slice>(store_elems.cbegin(), store_elems.cend()), false, &new_size);
+    if (!s.ok()) return s;
+  }
+
+  return rocksdb::Status::OK();
+}
+
+bool RedisSortObject::SortCompare(const RedisSortObject &a, const RedisSortObject &b, const SortArgument &args) {
+  if (!args.alpha) {
+    double score_a = std::get<double>(a.v);
+    double score_b = std::get<double>(b.v);
+    return !args.desc ? score_a < score_b : score_a > score_b;
+  } else {
+    if (!args.sortby.empty()) {
+      std::string cmp_a = std::get<std::string>(a.v);
+      std::string cmp_b = std::get<std::string>(b.v);
+      return !args.desc ? cmp_a < cmp_b : cmp_a > cmp_b;
+    } else {
+      return !args.desc ? a.obj < b.obj : a.obj > b.obj;
+    }
+  }
+}
+
 }  // namespace redis
