@@ -96,6 +96,7 @@ std::string InternalKey::Encode() const {
 }
 
 bool InternalKey::operator==(const InternalKey &that) const {
+  if (namespace_ != this->namespace_) return false;
   if (key_ != that.key_) return false;
   if (sub_key_ != that.sub_key_) return false;
   return version_ == that.version_;
@@ -471,21 +472,3 @@ rocksdb::Status JsonMetadata::Decode(Slice *input) {
 
   return rocksdb::Status::OK();
 }
-
-void SearchMetadata::Encode(std::string *dst) const {
-  Metadata::Encode(dst);
-
-  PutFixed8(dst, uint8_t(on_data_type));
-}
-
-rocksdb::Status SearchMetadata::Decode(Slice *input) {
-  if (auto s = Metadata::Decode(input); !s.ok()) {
-    return s;
-  }
-
-  if (!GetFixed8(input, reinterpret_cast<uint8_t *>(&on_data_type))) {
-    return rocksdb::Status::InvalidArgument(kErrMetadataTooShort);
-  }
-
-  return rocksdb::Status::OK();
-}
