@@ -19,9 +19,10 @@
 
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, REMAINDER
 from glob import glob
-from os import makedirs
+import os
 from pathlib import Path
 import re
+import filecmp
 from subprocess import Popen, PIPE
 import sys
 from typing import List, Any, Optional, TextIO, Tuple
@@ -92,6 +93,24 @@ def check_version(current: str, required: Tuple[int, int, int], prog_name: Optio
 
     return semver
 
+def prepare() -> None:
+    basedir = Path(__file__).parent.absolute()
+    
+    # Install Git hooks
+    hooks = basedir / "dev" / "hooks"
+    git_hooks = basedir / ".git" / "hooks"
+
+    git_hooks.mkdir(exist_ok=True)
+    for hook in hooks.iterdir():
+        dst = git_hooks / hook.name
+        if dst.exists():
+            if filecmp.cmp(hook, dst, shallow=False):
+                print(f"{hook.name} already installed.")
+                continue
+            raise RuntimeError(f"{dst} already exists; please remove it first")
+        else:
+            dst.symlink_to(hook)
+            print(f"{hook.name} installed at {dst}.")
 
 def build(dir: str, jobs: Optional[int], ghproxy: bool, ninja: bool, unittest: bool, compiler: str, cmake_path: str, D: List[str],
           skip_build: bool) -> None:
@@ -106,7 +125,7 @@ def build(dir: str, jobs: Optional[int], ghproxy: bool, ninja: bool, unittest: b
     cmake_version = output.read().strip()
     check_version(cmake_version, CMAKE_REQUIRE_VERSION, "CMake")
 
-    makedirs(dir, exist_ok=True)
+    os.makedirs(dir, exist_ok=True)
 
     cmake_options = ["-DCMAKE_BUILD_TYPE=RelWithDebInfo"]
     if ghproxy:
@@ -415,6 +434,13 @@ if __name__ == '__main__':
     parser_test_go.add_argument('rest', nargs=REMAINDER, help="the rest of arguments to forward to go test")
     parser_test_go.set_defaults(func=test_go)
 
+    parser_prepare = subparsers.add_parser(
+        'prepare',
+        description="Prepare scripts such as git hooks",
+        help="Prepare scripts such as git hooks"
+    )
+    parser_prepare.set_defaults(func=prepare)
+
     args = parser.parse_args()
 
     arg_dict = dict(vars(args))
